@@ -1,7 +1,7 @@
 /** @file
  * @brief Unit tests of non-Xapian-specific internal code.
  */
-/* Copyright (C) 2006,2007,2009,2010,2012,2015,2016,2018,2019 Olly Betts
+/* Copyright (C) 2006-2022 Olly Betts
  * Copyright (C) 2007 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
@@ -90,7 +90,7 @@ using namespace std;
 // fileutils.cc uses opendir(), etc though not in a function we currently test.
 #include "../common/msvc_dirent.cc"
 
-// The UUID code uses hexdigit().
+// The UUID code uses hex_digit().
 #include "../api/constinfo.cc"
 
 // Stub replacement, which doesn't deal with escaping or producing valid UTF-8.
@@ -744,19 +744,72 @@ static void test_movesupport1()
 }
 
 static void test_addoverflows1()
+{
+    const auto ulong_max = numeric_limits<unsigned long>::max();
+    const auto uint_max = numeric_limits<unsigned int>::max();
+    const auto ushort_max = numeric_limits<unsigned short>::max();
+    const auto uchar_max = numeric_limits<unsigned char>::max();
+
+    unsigned long res_ulong;
+    unsigned res_uint;
+    unsigned short res_ushort;
+    unsigned char res_uchar;
+
+    TEST(!add_overflows(0UL, 0UL, res_ulong));
+    TEST_EQUAL(res_ulong, 0);
+    TEST(!add_overflows(0UL, 0UL, res_uint));
+    TEST_EQUAL(res_uint, 0);
+    TEST(!add_overflows(0UL, 0UL, res_ushort));
+    TEST_EQUAL(res_ushort, 0);
+    TEST(!add_overflows(0UL, 0UL, res_uchar));
+    TEST_EQUAL(res_uchar, 0);
+
+    TEST(add_overflows(ulong_max, 1UL, res_ulong));
+    TEST_EQUAL(res_ulong, 0);
+    TEST(add_overflows(uint_max, 1UL, res_uint));
+    TEST_EQUAL(res_uint, 0);
+    TEST(add_overflows(ushort_max, 1UL, res_ushort));
+    TEST_EQUAL(res_ushort, 0);
+    TEST(add_overflows(uchar_max, 1UL, res_uchar));
+    TEST_EQUAL(res_uchar, 0);
+
+    TEST(add_overflows(1UL, ulong_max, res_ulong));
+    TEST_EQUAL(res_ulong, 0);
+    TEST(add_overflows(1UL, uint_max, res_uint));
+    TEST_EQUAL(res_uint, 0);
+    TEST(add_overflows(1UL, ushort_max, res_ushort));
+    TEST_EQUAL(res_ushort, 0);
+    TEST(add_overflows(1UL, uchar_max, res_uchar));
+    TEST_EQUAL(res_uchar, 0);
+
+    TEST(add_overflows(ulong_max, ulong_max, res_ulong));
+    TEST_EQUAL(res_ulong, ulong_max - 1UL);
+    TEST(add_overflows(uint_max, uint_max, res_uint));
+    TEST_EQUAL(res_uint, uint_max - 1UL);
+    TEST(add_overflows(ushort_max, ushort_max, res_ushort));
+    TEST_EQUAL(res_ushort, ushort_max - 1UL);
+    TEST(add_overflows(uchar_max, uchar_max, res_uchar));
+    TEST_EQUAL(res_uchar, uchar_max - 1UL);
+
+    res_uchar = 1;
+    TEST(add_overflows(res_uchar, unsigned(uchar_max) + 1U, res_uchar));
+    TEST_EQUAL(res_uchar, 1);
+}
+
+static void test_suboverflows1()
 {
     unsigned long res;
-    TEST(!add_overflows(0UL, 0UL, res));
+    TEST(!sub_overflows(0UL, 0UL, res));
     TEST_EQUAL(res, 0);
 
-    TEST(add_overflows(ULONG_MAX, 1UL, res));
-    TEST_EQUAL(res, 0);
+    TEST(sub_overflows(0UL, 1UL, res));
+    TEST_EQUAL(res, ULONG_MAX);
 
-    TEST(add_overflows(1UL, ULONG_MAX, res));
-    TEST_EQUAL(res, 0);
+    TEST(sub_overflows(ULONG_MAX - 1UL, ULONG_MAX, res));
+    TEST_EQUAL(res, ULONG_MAX);
 
-    TEST(add_overflows(ULONG_MAX, ULONG_MAX, res));
-    TEST_EQUAL(res, ULONG_MAX - 1UL);
+    TEST(sub_overflows(0UL, ULONG_MAX, res));
+    TEST_EQUAL(res, 1);
 }
 
 static void test_muloverflows1()
@@ -872,6 +925,7 @@ static const test_desc tests[] = {
     TESTCASE(uuid1),
     TESTCASE(movesupport1),
     TESTCASE(addoverflows1),
+    TESTCASE(suboverflows1),
     TESTCASE(muloverflows1),
     TESTCASE(parseunsigned1),
     TESTCASE(parsesigned1),
